#include "lib/std.mi"

Global Text Clock;
Global Timer clockTimer;
Global Int showseconds;

System.onScriptLoaded() {
  Group thisGroup = getScriptGroup();
  Clock = thisGroup.getObject("Clock");

  // Retrieve the saved value of the showseconds variable:
  showseconds = getPrivateInt("this skin", "showseconds", showseconds);

  // Initialize the timer:
  clockTimer = new Timer;
  clockTimer.setDelay(100);
  clockTimer.start();
}

// Toggle showseconds on and off:
Clock.OnLeftButtonup(int X, int Y) {
  if (showseconds == 0) {
    showseconds = 1;
    // Show the following tooltip when hovering over the clock:
    Clock.setXmlParam("tooltip", "Click to hide seconds");
  }
  else {
    showseconds = 0;
    // Show the following tooltip when hovering over the clock:
    Clock.setXmlParam("tooltip", "Click to show seconds");
  }
}

clockTimer.onTimer() {
  // The getTimeOfDay() function returns the time passed since midnight
  // in milliseconds.
  // Convert the milliseconds to HH:MM:SS
  String HHMMSS = integerToLongTime(getTimeOfDay());
  // Remove the last three characters of HH:MM:SS
  String HHMM = strleft(HHMMSS, strlen(HHMMSS)-3);
  // If the value of showseconds is 0 then don't show the seconds:
  if (showseconds == 0) Clock.setText(HHMM);
  // Otherwise show them:
  else Clock.setText(HHMMSS);
}

System.onScriptUnloading() {
  delete clockTimer;
  setPrivateInt("this skin", "showseconds", showseconds);
}
