/* coversize.m

Author:	Ariszló <ariszlo@gmail.com>

Date:	2021-08-17

HOW TO USE

1. Add a button like this to your coverart.xml replacing the question marks
   with actual values:

   <button
      id="Cover Size"
      x="?" y="?" w="?" h="?"
      image="?"
      tooltip="Select cover size"
   />

2. Add default_w and default_h parameters to the layout tag:

   <layout id="normal" background="?" default_w="?" default_h="?">

   <!-- default_w = 300 + the widths of frame.left.stretch & frame.right.stretch
      default_h = 300 + the heights of frame.top.stretch & frame.bottom.stretch
      Find the width & height values in standardframe.xml
   -->

3. Add this after the layout tag:

   <script id="coversize" file="scripts/coversize.maki"/>

4. Copy coversize.m and coversize.maki into the scripts folder. */

#include "lib/std.mi"

Global Container coverartContainer;
Global Layout coverartLayout;
Global String defaultw, defaulth;
Global Button coversizeButton;

Function coversizePopup();

System.onScriptLoaded() {
  coverartContainer = System.getContainer("coverart");
  coverartLayout = coverartContainer.getLayout("normal");
  defaultw = coverartLayout.getXMLparam("default_w");
  defaulth = coverartLayout.getXMLparam("default_h");
  coversizeButton = coverartLayout.findObject("Cover Size");
}

coversizeButton.onLeftButtonUp(int X, int Y) { coversizePopup(); }

coversizePopup() {
  PopupMenu sizeMenu;

  sizeMenu = new PopupMenu;
  sizeMenu.addCommand("300x300", 1,0,0);
  sizeMenu.addCommand("600x600", 2,0,0);
  sizeMenu.addCommand("900x900", 3,0,0);

  if (sizeMenu == Null) { return; }
  Int sizeNumber = sizeMenu.popAtMouse();
  if (sizeNumber > 0) {
    if (sizeNumber == 1) {
      coverartLayout.setXmlParam("w", defaultw);
      coverartLayout.setXmlParam("h", defaulth);
    }
    if (sizeNumber == 2) {
      String w600 = IntegerToString(StringToInteger(defaultw)+300);
      String h600 = IntegerToString(StringToInteger(defaulth)+300);
      coverartLayout.setXmlParam("w", w600);
      coverartLayout.setXmlParam("h", h600);
    }
    if (sizeNumber == 3) {
      String w900 = IntegerToString(StringToInteger(defaultw)+600);
      String h900 = IntegerToString(StringToInteger(defaulth)+600);
      coverartLayout.setXmlParam("w", w900);
      coverartLayout.setXmlParam("h", h900);
    }
  }
  complete;

  delete sizeMenu;
}
