/* >>> Use either this script or Bento's songticker.maki for your skin
       BUT NOT BOTH! <<<

tickerpopup.m

Version 1.1 (2022-03-21) by Ariszló <ariszlo@gmail.com>
// cleaned up & added comments

Left-click on the song ticker to pop up a menu of scrolling options.

HOW TO USE

1. Add these tags to your xml replacing the question marks with the actual values:

  <SongTicker
    id="SongTicker"
    ticker="bounce" showlen="1"
    x="?" y="?" w="?" h="?"
    rightclickaction="trackmenu"
    tooltip="Left-click to change ticker scrolling"
  />
  <script id="tickerpopup" file="scripts/tickerpopup.maki"/>

2. Copy tickerpopup.m and tickerpopup.maki into the scripts folder. */

#include "lib/std.mi"
#include "lib/config.mi"

// Declare your global variables:
Global ConfigItem scrollConfig;
Global ConfigAttribute scrollAttrib;
Global GuiObject songticker;

// Do this when the script is loaded:
System.onScriptLoaded() {
  // Reuse custom_page_nonexposed from Bento's gen_pageguids.m for scrollConfig:
  scrollConfig = Config.newItem("Hidden","{E9C2D926-53CA-400f-9A4D-85E31755A4CF}");
  // Define the default scroll attribute:
  scrollAttrib = scrollConfig.newAttribute("scrollAttrib","bounce");

  // Find the songticker object within the xml group containing both
  // the songticker tag and the script tag calling the present script:
  Group scriptGroup = getScriptGroup();
  songticker = scriptGroup.getObject("SongTicker");

  // Use the stored scroll attribute (or fall back to the default):
  songticker.setXmlParam("ticker",scrollAttrib.getData());
}

// What should happen when you left-click on the song ticker?
songticker.onLeftButtonUp(int x, int y) {
  // Create a new popup menu by first giving it a name and then creating it:
  PopupMenu scrollMenu;
  scrollMenu = new PopupMenu;

  // List the options:
  // 1st digit = option id
  // 2nd digit: 1 = active
  // 3rd digit: 1 = disabled

  // Show this popup when "Scroll both directions" is active:
  if (scrollAttrib.getData() == "bounce") {
    scrollMenu.addCommand("Scroll both directions", 1,1,0);
    scrollMenu.addCommand("Scroll right to left", 2,0,0);
    scrollMenu.addCommand("Do not scroll", 3,0,0);
  }
  // Show this when "Scroll right to left" is active:
  if (scrollAttrib.getData() == "scroll") {
    scrollMenu.addCommand("Scroll both directions", 1,0,0);
    scrollMenu.addCommand("Scroll right to left", 2,1,0);
    scrollMenu.addCommand("Do not scroll", 3,0,0);
  }
  // Show this when scrolling is disabled:
  if (scrollAttrib.getData() == "off") {
    scrollMenu.addCommand("Scroll both directions", 1,0,0);
    scrollMenu.addCommand("Scroll right to left", 2,0,0);
    scrollMenu.addCommand("Do not scroll", 3,1,0);
  }

  // Quit the script in the unlikely event when the popup fails:
  if (scrollMenu == Null) { return; }

  // Get the clicked option's id:
  Int scrollNumber = scrollMenu.popAtMouse();

  // Make the clicked option active:
  if (scrollNumber > 0) {
    if (scrollNumber == 1) {
      // Set the ticker attribute for now:
      songticker.setXmlParam("ticker", "bounce");
      // Make Winamp remember it at the next startup:
      scrollAttrib.setData("bounce");
    }
    if (scrollNumber == 2) {
      songticker.setXmlParam("ticker", "scroll");
      scrollAttrib.setData("scroll");
    }
    if (scrollNumber == 3) {
      songticker.setXmlParam("ticker", "off");
      scrollAttrib.setData("off");
    }
  }

  complete;
  delete scrollMenu;
}

