/* times.m by Ariszlo for the Times demo skin, 2023-06-18 */

#include "lib/std.mi"

Global Text toggleTime, songTime, Milliseconds;
Global Timer trackTimer;
Global Int timeleft;

System.onScriptLoaded() {
  group thisGroup = getScriptGroup();

  toggleTime = thisGroup.getObject("Toggle Time");
  songTime = thisGroup.getObject("Song Times");
  Milliseconds = thisGroup.getObject("Milliseconds");

  // Retrieve the value of timeleft:
  timeleft = getPrivateInt("this skin", "timeleft", timeleft);
  // Set the default value of timeleft to 0:
  if (timeleft != 1) timeleft = 0;

  trackTimer = new Timer;
  trackTimer.setDelay(100);

  if (getStatus() != 0) {
      if (getPlayItemstring() != "") trackTimer.start();
  }
}

System.onPlay() {
  if (getPlayItemstring() != "") trackTimer.start();
}

System.onPause() {
  if (getPlayItemstring() != "") {
    trackTimer.stop();
    trackTimer.start();
  }
}

System.onResume() {
  if (getPlayItemstring() != "") trackTimer.start();
}

// Toggle elapsed time and remaining time by clicking on the "Toggle Time"
// text object:
toggleTime.OnLeftButtonup(int X, int Y) {
  if (timeleft == 0) timeleft = 1;
  else timeleft = 0;
}

trackTimer.onTimer() {
  // The getPosition() function returns elapsed time in milliseconds.
  // Round elapsed time to the nearest thousand:
  String elapsedString = floatToString(getPosition()/1000, 1);
  Int elapsedLeft = StringToInteger(getToken(elapsedString, "." , 0));
  Int elapsedRight = StringToInteger(getToken(elapsedString, "." , 1));
  Int roundElapsed;
  if (elapsedRight >= 5) {roundElapsed = (elapsedLeft + 1)*1000;}
  else {roundElapsed = elapsedLeft*1000;}

  // By default, show elapsed time in the "Toggle Time" text object:
  if (timeleft==0) {
    // If the song is shorter than 1 hour then only show minutes and seconds:
    if (getPlayItemLength() < 3600000) {
      toggleTime.setXmlParam("y", "174");
      toggleTime.setXmlParam("h", "111");
      toggleTime.setXmlParam("fontsize", "111");
      toggleTime.setText(integerToTime(roundElapsed));
    }
    // If longer then show hours, minutes and seconds:
    else {
      toggleTime.setXmlParam("y", "196");
      toggleTime.setXmlParam("h", "84");
      toggleTime.setXmlParam("fontsize", "84");
      toggleTime.setText(integerToLongTime(roundElapsed));
    }
    // Keep the value of the timeleft variable at 0:
    timeleft=0;
  }

  // getPlayItemLength() returns the length of the song in milliseconds,
  // always rounded down to the nearest thousand (no need to round it):
  if (getPlayItemLength() > 0) {
    // Calculate remaining time:
    Int roundRemaining = getPlayItemLength()-roundElapsed;

    // Show remaining time in the "Toggle Time" text object if toggled to do so:
    if (timeleft==1) {
      if (getPlayItemLength() < 3600000) {
        toggleTime.setXmlParam("y", "174");
        toggleTime.setXmlParam("h", "111");
        toggleTime.setXmlParam("fontsize", "111");
        toggleTime.setText("-" + integerToTime(roundRemaining));
      }
      else {
        toggleTime.setXmlParam("y", "196");
        toggleTime.setXmlParam("h", "84");
        toggleTime.setXmlParam("fontsize", "84");
        toggleTime.setText("-" + integerToLongTime(roundRemaining));
      }
      // Keep the value of the timeleft variable at 1:
      timeleft=1;
    }

    // Show elapsed time, remaining time and song length in milliseconds:
    Milliseconds.setText(integerToString(getPosition()) + "  -" + integerToString(getPlayItemLength()-getPosition()) + "  " + integerToString(getPlayItemLength()));

    // Show elapsed time, remaining time and song length in (HH:):MM:SS format:
    if (getPlayItemLength() < 3600000) {
      songTime.setText(integerToTime(roundElapsed) + "  -" + integerToTime(roundRemaining) + "  " + integerToTime(getPlayItemLength()) );
    }
    else {
      songTime.setText(integerToLongTime(roundElapsed) + "  -" + integerToLongTime(roundRemaining) + "  " + integerToLongTime(getPlayItemLength()) );
    }
  }
  // If song length cannot be determined then only show elapsed time:
  else {
    // Show elapsed time in the "Toggle Time" text object even if it is
    // toggled to show remaining time:
    if (timeleft==1) {
      if (getPlayItemLength() < 3600000) {
        toggleTime.setXmlParam("y", "174");
        toggleTime.setXmlParam("h", "111");
        toggleTime.setXmlParam("fontsize", "111");
        toggleTime.setText(integerToTime(roundElapsed));
      }
      else {
        toggleTime.setXmlParam("y", "196");
        toggleTime.setXmlParam("h", "84");
        toggleTime.setXmlParam("fontsize", "84");
        toggleTime.setText(integerToLongTime(roundElapsed));
      }
      // Change the value of the timeleft variable back to 0:
      timeleft=0;
    }
    // Show elapsed time in milliseconds:
    Milliseconds.setText(integerToString(getPosition()));

    // Show elapsed time in (HH:):MM:SS format:
    if (getPlayItemLength() < 3600000) {
      songTime.setText(integerToTime(roundElapsed));
    }
    else {
      songTime.setText(integerToLongTime(roundElapsed));
    }
  }
}

System.onScriptUnloading() {
  delete trackTimer;
  setPrivateInt("this skin", "timeleft", timeleft);
}
